# -*- coding: utf-8 -*-

# This plugin is free software, you are allowed to
# modify it (if you keep the license),
# but you are not allowed to distribute/publish
# it without source code (this version and your modifications).
# This means you also have to distribute
# source code of your modifications.

# maintainer: <plnick@vuplus-support.org>

# modified by RP (remo66) <dev@running-egg.de>
#
# 2020-03-06
# * A/V audio activation (setup, wakeup (standby/deepstandby) and delayedActiveSource)
# * Logging improved
# 2021-01-07
# * Logging improved (message payload added / some fixes)
# * CEC device discovery added
# 2021-01-22
# * dictionaries for CEC version and vendor added
# 2021-01-22
# * dictionaries for device type and power status added
# 2021-02-17
# * wrapper for eHDMIcec.SendMessage calls with logging
# 2021-02-24
# moved optional A/V audio activation (avaudioon) into wakup configuration
# * "delayedActiveSource" (including additional messages) handling rewitten (now: managed by delayedmessages_queue)
# * changed http port config from text to integer

# Todo:
#  * move "AV activation" from separate action into "AV power" action???
#  * use Polling message (instead of power status message) for device discovery
#  * add VU+ to device list
#  * do not set logical address by constant, retrieve from configuration???

import struct
from enigma import eHdmiCEC, eTimer
from Components.config import config, ConfigSelection, ConfigYesNo, ConfigSubsection, ConfigText, ConfigInteger
from Components.Language import language
from Tools.HardwareInfo import HardwareInfo
from Screens.Standby import inStandby
import Screens.Standby
from Tools import Notifications
from Tools.DreamboxHardware import getFPWasTimerWakeup
from Tools.Directories import fileExists
import time
from datetime import datetime


class HdmiCec:
	ADDR_TV = int("0x00", 16)
	ADDR_RECORDER1 = int("0x01", 16)
	ADDR_RECORDER2 = int("0x02", 16)
	ADDR_TUNER1 = int("0x03", 16)  # default for the VU+ device
	ADDR_PLAYBACK1 = int("0x04", 16)
	ADDR_AUDIOSYSTEM = int("0x05", 16)
	ADDR_TUNER2 = int("0x06", 16)
	ADDR_TUNER3 = int("0x07", 16)
	ADDR_PLAYBACK2 = int("0x08", 16)
	ADDR_RECORDER3 = int("0x09", 16)
	ADDR_TUNER4 = int("0x0A", 16)
	ADDR_PLAYBACK3 = int("0x0B", 16)
	ADDR_RESERVED1 = int("0x0C", 16)
	ADDR_RESERVED2 = int("0x0D", 16)
	ADDR_SPECIFICUSE = int("0x0E", 16)
	ADDR_BROADCAST = int("0x0F", 16)

	ADDR_SELF = ADDR_RECORDER1  # Todo: looks like "Recording1" is default for Enigma2 devices, but retrieving is better

	MSG_POLL = int("0x00", 16)
	MSG_FEATUREABORT = int("0x00", 16)
	MSG_WAKEUP = int("0x04", 16)
	MSG_STANDBY = int("0x36", 16)
	MSG_SENDKEY = int("0x44", 16)
	MSG_GIVENAME = int('0x46', 16)
	MSG_SETNAME = int('0x47', 16)
	MSG_SENDKEYPWRON = int("0x6D", 16)
	MSG_SETOSDSTRING = int("0x64", 16)
	MSG_SENDKEYPWROFF = int("0x6C", 16)
	MSG_AUDIOREQ = int("0x70", 16)
	MSG_ACTIVESOURCE = int("0x82", 16)
	MSG_GIVEPHYSADDRESS = int('0x83', 16)
	MSG_PHYSADDRESS = int('0x84', 16)
	MSG_REQUESTACTIVESOURCE = int("0x85", 16)
	MSG_VENDORID = int("0x87", 16)
	MSG_MENUSTATUS = int('0x8E', 16)
	MSG_GIVEVENDORID = int("0x8C", 16)
	MSG_GIVEPOWERSTATUS = int("0x8F", 16)
	MSG_INACTIVESOURCE = int("0x9D", 16)
	MSG_REPORTPOWERSTATUS = int("0x9D", 16)
	MSG_GETMENULANGUAGE = int("0x91", 16)
	MSG_CECVERSION = int("0x9E", 16)
	MSG_GETCECVERSION = int("0x9F", 16)

	TYPE_TV = int('0x00', 16)
	TYPE_RECORDING = int('0x01', 16)
	TYPE_RESERVED = int('0x02', 16)
	TYPE_TUNER = int('0x03', 16)
	TYPE_PLAYBACK = int('0x04', 16)
	TYPE_AUDIO = int('0x01', 16)

	TYPE_SELF = TYPE_RECORDING  # Todo: looks like "Recording" is default for Enigma2 devices, but retrieving is better

	# CECVER_SELF = int('0x04', 16)  # original plugin
	CECVER_SELF = int('0x05', 16)  # https://en.wikipedia.org/wiki/Vu%2B
	VENDORID_SELF = int('0x00', 16)

	POWER_ON = int("0x00", 16)
	POWER_OFF = int("0x01", 16)

	MENU_ON = int('0x00', 16)
	MENU_OFF = int('0x01', 16)

	CECtypes = {
		0x00: "TV",
		0x01: "Recording",
		0x02: "Reserved",
		0x03: "Tuner",
		0x04: "Playback",
		0x05: "Audio"
	}

	CECstatus = {
		0x00: "on",
		0x01: "off",
		0x02: ">on",
		0x03: ">off"
	}

	CECversions = {
		0x00: "unknown",
		0x01: "1.2",
		0x02: "1.2A",
		0x03: "1.3",
		0x04: "1.3A",
		0x05: "1.4",
		0x06: "2.0"
	}
	
	CECvendors = {
		0x0: "UNKNOWN",
		0x000039: "TOSHIBA",
		0x0000F0: "SAMSUNG",
		0x0005CD: "DENON",
		0x000678: "MARANTZ",
		0x000982: "LOEWE",
		0x0009B0: "ONKYO",
		0x000CB8: "MEDION",
		0x000CE7: "TOSHIBA/AMAZON",  # TOSHIBA2
		0x0010FA: "APPLE",
		0x001582: "PULSE_EIGHT",
		0x001950: "HARMAN/KARDON",  # HARMAN/KARDON2
		0x001A11: "GOOGLE",
		0x0020C7: "AKAI",
		0x002467: "AOC",
		0x008045: "PANASONIC",
		0x00903E: "PHILIPS",
		0x009053: "DAEWOO",
		0x00A0DE: "YAMAHA",
		0x00D0D5: "GRUNDIG",
		0x00E036: "PIONEER",
		0x00E091: "LG",
		0x08001F: "SHARP",
		0x080046: "SONY",
		0x18C086: "BROADCOM",
		0x534850: "SHARP2",
		0x6B746D: "VIZIO",
		0x8065E9: "BENQ",
		0x9C645E: "HARMAN/KARDON"
	}

	DevList = {}
	
	def __init__(self):
		config.hdmicec = ConfigSubsection()
		config.hdmicec.enabled = ConfigYesNo(default=False)
		config.hdmicec.logenabledserial = ConfigYesNo(default=False)
		config.hdmicec.logenabledfile = ConfigYesNo(default=False)
		config.hdmicec.tvstandby = ConfigYesNo(default=False)
		config.hdmicec.tvwakeup = ConfigYesNo(default=False)
		config.hdmicec.boxstandby = ConfigYesNo(default=False)
		config.hdmicec.enabletvrc = ConfigYesNo(default=True)
		config.hdmicec.active_source_reply = ConfigYesNo(default=True)
		config.hdmicec.avvolumecontrol = ConfigYesNo(default=False)
		# config.hdmicec.avonwakeup = ConfigYesNo(default=False)
		config.hdmicec.disabletimerwakeup = ConfigYesNo(default=True)
		config.hdmicec.expert_enabled = ConfigYesNo(default=False)
		config.hdmicec.http_enabled = ConfigYesNo(default=False)
		config.hdmicec.testing_enabled = ConfigYesNo(default=False)
		config.hdmicec.device_name = ConfigText(default=self.getDeviceName(), visible_width=50, fixed_size=False)
		config.hdmicec.testmsg = ConfigText(default="00", visible_width=50, fixed_size=False)
		config.hdmicec.testdata = ConfigText(default="", visible_width=50, fixed_size=False)
		config.hdmicec.http_port = ConfigInteger(default=8000, limits=(1, 99999))
		config.hdmicec.standby_message = ConfigSelection(default="standby,inactive", choices=[
			("standby,inactive", _("TV standby")),
			("standby,avpwroff,inactive,", _("TV + A/V standby")),
			("inactive", _("Source inactive")),
			("nothing", _("Nothing"))])
		config.hdmicec.deepstandby_message = ConfigSelection(default="standby,inactive", choices=[
			("standby,inactive", _("TV standby")),
			("standby,avdeeppwroff,inactive", _("TV + A/V standby")),
			("inactive", _("Source inactive")),
			("nothing", _("Nothing"))])
		config.hdmicec.wakeupstandby_message = ConfigSelection(default="wakeup,active,activevu", choices=[
			("wakeup,active,activevu", _("TV wakeup")),
			("wakeup,active,activevu,avpwron", _("TV + A/V wakeup")),
			("wakeup,active,activevu,avpwron,avaudioon", _("TV + A/V wakeup (A/V audio)")),
			("avpwron,wakeup,active,activevu", _("A/V + TV wakeup")),
			("avpwron,wakeup,active,activevu,avaudioon", _("A/V + TV wakeup (A/V audio)")),
			("active", _("Source active")),
			("nothing", _("Nothing"))])
		config.hdmicec.wakeupdeepstandby_message = ConfigSelection(default="wakeup,active,activevu", choices=[
			("wakeup,active,activevu", _("TV wakeup")),
			("wakeup,active,activevu,avpwron", _("TV + A/V wakeup")),
			("wakeup,active,activevu,avpwron,avaudioon", _("TV + A/V wakeup (A/V audio)")),
			("avpwron,wakeup,active,activevu", _("A/V + TV wakeup")),
			("avpwron,wakeup,active,activevu,avaudioon", _("A/V + TV wakeup (A/V audio)")),
			("active", _("Source active")),
			("nothing", _("Nothing"))])
		config.hdmicec.vustandby_message = ConfigSelection(default="vustandby", choices=[
			("vustandby", _("VU+ standby")),
			("vudeepstandby", _("VU+ DeepStandby")),
			("vunothing", _("Nothing"))])
		config.hdmicec.vuwakeup_message = ConfigSelection(default="vuwakeup", choices=[
			("vuwakeup", _("VU+ wakeup")),
			("vunothing", _("Nothing"))])
		config.hdmicec.tvinput = ConfigSelection(default="1", choices=[
			("1", "HDMI 1"),
			("2", "HDMI 2"),
			("3", "HDMI 3"),
			("4", "HDMI 4"),
			("5", "HDMI 5"),
			("6", "HDMI 6"),
			("7", "HDMI 7")])
		config.hdmicec.avinput = ConfigSelection(default="0", choices=[
			("0", _("no A/V Receiver")),
			("1", "HDMI 1"),
			("2", "HDMI 2"),
			("3", "HDMI 3"),
			("4", "HDMI 4"),
			("5", "HDMI 5"),
			("6", "HDMI 6"),
			("7", "HDMI 7")])
		config.hdmicec.message_delay = ConfigSelection(default="10", choices=[
			("1", "0.1 sec"),
			("5", "0.5 sec"),
			("10", "1 sec"),
			("20", "2 sec"),
			("30", "3 sec"),
			("50", "5 sec")])
		config.hdmicec.testaddr = ConfigSelection(default="0", choices=[
			("00", "0 (TV)"),
			("01", "1 (Recording 1)"),
			("02", "2 (Recording 2)"),
			("03", "3 (Tuner 1)"),
			("04", "4 (Playback 1)"),
			("05", "5 (Audio system)"),
			("06", "6 (Tuner 2)"),
			("07", "7 (Tuner 3)"),
			("08", "8 ( Playback 2)"),
			("09", "9 (Recording 3)"),
			("0A", "A (Tuner 4)"),
			("0B", "B (Playback 3)"),
			("0C", "C (Reserved 1)"),
			("0D", "D (Reserved 2)"),
			("0E", "E (Specific use)"),
			("0F", "F (Broadcast)")])
		self.cecmessage_queue = []
		self.cecdelayedmessage_queue = []
		self.delayTimer = eTimer()
		self.delayTimer.callback.append(self.sendCECMessage)
		self.delayTimer_intervall = int(config.hdmicec.message_delay.value) * 100
		self.delayed_Message_Timer = None
		config.misc.standbyCounter.addNotifier(self.enterStandby, initial_call=False)
		config.misc.DeepStandbyOn.addNotifier(self.enterDeepStandby, initial_call=False)
		self.ActiveSource = False
		self.initial_active_source_call = True
		self.activeSourceTimer = None
		self.activateSourceTimer()
		self.leaveDeepStandby()

	def doLog(self, txt):
		now = datetime.now()  # current date and time
		date_time = now.strftime("%Y-%m-%d/%H:%M:%S")
		logtxt = "[VTI HDMI-CEC] %s: %s\n" % (date_time, txt)
		if config.hdmicec.logenabledserial.value:
			vtilog(logtxt)
		if config.hdmicec.logenabledfile.value:
			self.writeFile("/tmp/hdmicec.log", "a", logtxt)

	@staticmethod
	def writeFile(filename, mode, line):
		try:
			with open(filename, mode) as f:
				f.write(line)
		except Exception, e:
			txt = "[VTI HDMI-CEC] write file '%s' failed - error: %s" % (file, e)
			print txt

	@staticmethod
	def getDeviceName():
		return "Vu+ " + HardwareInfo().get_friendly_name()

	@staticmethod
	def getConfigDeviceName():
		name_len = len(config.hdmicec.device_name.value)
		if name_len == 0:
			return "VU+"
		else:
			return config.hdmicec.device_name.value

	@staticmethod
	def getDeviceLanguage():
		lang = language.getLanguage()
		return lang[:2]

	@staticmethod
	def getPhysicalAddress():
		pa = "%s.%s.0.0" % (str(config.hdmicec.tvinput.value), str(config.hdmicec.avinput.value))
		return pa

	def getActiveSource(self):
		if self.ActiveSource:
			return "X"
		else:
			return ""

	def getDevicePower(self):
		if config.hdmicec.enabled.value:
			from Screens.Standby import inStandby
		power = self.POWER_ON
		if inStandby and (config.hdmicec.vuwakeup_message.value == "vuwakeup"):
			power = self.POWER_OFF
		return self.CECstatus[power]

	def setDeviceInfo(self):
		# pa = eHdmiCEC.getInstance().getPhysicalAddress()
		# la = eHdmiCEC.getInstance().getLogicalAddress()
		# dt = eHdmiCEC.getInstance().getDeviceType()
		la = self.ADDR_SELF
		name = self.getConfigDeviceName()
		# vendor = self.CECvendors[self.VENDORID_SELF]
		vendor = "VU+/MARUSYS"
		dt = self.CECtypes[self.TYPE_SELF]
		ver = self.CECversions[self.CECVER_SELF]
		lng = self.getDeviceLanguage()
		pa = self.getPhysicalAddress()
		active = self.getActiveSource()
		power = self.getDevicePower()
		if config.hdmicec.logenabledserial.value or config.hdmicec.logenabledfile.value:
			self.doLog("setDeviceInfo: %s,%s,%s,%s,%s,%s,%s,%s,%s" % (la, name, vendor, dt, ver, lng, pa, power, active))
		self.updateDeviceList(la, name, vendor, dt, ver, lng, pa, power, active)

	def discoverDevices(self):
		try:
			if config.hdmicec.logenabledserial.value or config.hdmicec.logenabledfile.value:
				self.doLog("discoverDevices...")
			self.DevList.clear()
			self.setDeviceInfo()

			# request active source by broadcast
			address = self.ADDR_BROADCAST
			cmd = struct.pack('B', self.MSG_REQUESTACTIVESOURCE)
			logcmd = "request active source from %02X (broadcast)" % address
			self.cecmessage_queue.append((cmd, address, logcmd))

			# go though all logical addresses, except broadcast address (15/0F)
			for i in range(15):
				address = int("0x%02X" % i, 16)
				# polling message
				# cmd = struct.pack('B', self.MSG_POLL)
				# logcmd = "** Send polling message to %02X" % (address)
				# self.cecmessage_queue.append((cmd, address, logcmd))
				# alternative polling by requesting power status
				cmd = struct.pack('B', self.MSG_GIVEPOWERSTATUS)
				logcmd = "request power status from %02X" % address
				self.cecmessage_queue.append((cmd, address, logcmd))
		except Exception as E:
			self.doLog("[HDMI-CEC] discoverDevices: " + E.message)

	def discoverDevice(self, address):
		try:
			if config.hdmicec.logenabledserial.value or config.hdmicec.logenabledfile.value:
				self.doLog("discoverDevice at %02X ..." % address)
			# Power Status
			# cmd = struct.pack('B', self.MSG_GIVEPOWERSTATUS)
			# logcmd = "** Request power status from %02X" % (address)
			# self.cecmessage_queue.append((cmd, address, logcmd))
			# power status
			cmd = struct.pack('B', self.MSG_GIVEPOWERSTATUS)
			logcmd = "request power status from %02X" % address
			self.cecmessage_queue.append((cmd, address, logcmd))
			# vendor ID
			cmd = struct.pack('B', self.MSG_GIVEVENDORID)
			logcmd = "request vendor ID with message %02X from %02X" % (
				self.MSG_GIVEVENDORID, address)
			self.cecmessage_queue.append((cmd, address, logcmd))
			# device name
			cmd = struct.pack('B', self.MSG_GIVENAME)
			logcmd = "request device name with message %02X from %02X" % (
				self.MSG_GIVENAME, address)
			self.cecmessage_queue.append((cmd, address, logcmd))
			# CEC version
			cmd = struct.pack('B', self.MSG_GETCECVERSION)
			logcmd = "request device CEC version with message %02X from %02X" % (
				self.MSG_GETCECVERSION, address)
			self.cecmessage_queue.append((cmd, address, logcmd))
			# Menu Language
			cmd = struct.pack('B', self.MSG_GETMENULANGUAGE)
			logcmd = "request device menu language with message %02X from %02X" % (
				self.MSG_GETMENULANGUAGE, address)
			self.cecmessage_queue.append((cmd, address, logcmd))
			# Physical Address (and Device Type)
			cmd = struct.pack('B', self.MSG_GIVEPHYSADDRESS)
			logcmd = "request device physical address (and device type) with message %02X from %02X" % (
				self.MSG_GIVEPHYSADDRESS, address)
			self.cecmessage_queue.append((cmd, address, logcmd))
			# send queued messages
			self.sendCECMessage(True)

		except Exception as E:
			self.doLog("[HDMI-CEC] discoverDevice: " + E.message)

	def updateDeviceList(self, la, name="", vendor="", devtype="", ver="", lng="", pa="", power="", active=""):
		try:
			if la in self.DevList:

				if len(name) > 0:
					if config.hdmicec.logenabledserial.value or config.hdmicec.logenabledfile.value:
						self.doLog("updateDeviceList: update %02X name=%s" % (la, name))
					self.DevList[la][0] = name
				if len(vendor) > 0:
					if config.hdmicec.logenabledserial.value or config.hdmicec.logenabledfile.value:
						self.doLog("updateDeviceList: update %02X vendor=%s" % (la, vendor))
					self.DevList[la][1] = vendor
				if len(devtype) > 0:
					if config.hdmicec.logenabledserial.value or config.hdmicec.logenabledfile.value:
						self.doLog("updateDeviceList: update %02X device type=%s" % (la, devtype))
					self.DevList[la][2] = devtype
				if len(ver) > 0:
					if config.hdmicec.logenabledserial.value or config.hdmicec.logenabledfile.value:
						self.doLog("updateDeviceList: update %02X version=%s" % (la, ver))
					self.DevList[la][3] = ver
				if len(lng) > 0:
					if config.hdmicec.logenabledserial.value or config.hdmicec.logenabledfile.value:
						self.doLog("updateDeviceList: update %02X language=%s" % (la, lng))
					self.DevList[la][4] = lng
				if len(pa) > 0:
					if config.hdmicec.logenabledserial.value or config.hdmicec.logenabledfile.value:
						self.doLog("updateDeviceList: update %02X physical address=%s" % (la, pa))
					self.DevList[la][5] = pa
				if len(power) > 0:
					if config.hdmicec.logenabledserial.value or config.hdmicec.logenabledfile.value:
						self.doLog("updateDeviceList: update %02X power status=%s" % (la, power))
					self.DevList[la][6] = power
				if len(active) > 0:
					if config.hdmicec.logenabledserial.value or config.hdmicec.logenabledfile.value:
						self.doLog("updateDeviceList: update %02X active source=%s" % (la, active))
					for key in self.DevList:
						self.DevList[key][7] = ""  # clear active source for all devices in list
					self.DevList[la][7] = active
			else:
				if config.hdmicec.logenabledserial.value or config.hdmicec.logenabledfile.value:
					self.doLog("updateDeviceList: add %02X" % la)
				self.DevList[la] = [name, vendor, devtype, ver, lng, pa, power, active]
		except Exception as E:
			self.doLog("[HDMI-CEC] updateDeviceList: " + E.message)

	def sendMessages(self, messages, delay=True):
		# messagedelay = float(config.hdmicec.message_delay.value)/10.0
		for message in messages.split(','):
			cmd = None
			logcmd = None
			physaddress1 = int("0x" + str(config.hdmicec.tvinput.value) + str(config.hdmicec.avinput.value), 16)
			physaddress2 = int("0x00", 16)
			avphysaddress1 = int("10", 16)
			avphysaddress2 = int("00", 16)
			AddressValue = int("0", 16)

			if message == "wakeup":
				AddressValue = self.ADDR_TV
				cmd = struct.pack('B', self.MSG_WAKEUP)
				logcmd = "** WakeUpMessage ** send message: %02X to address %02X" % (self.MSG_WAKEUP, AddressValue)
			elif message == "active":
				AddressValue = self.ADDR_BROADCAST
				cmd = struct.pack('BBB', self.MSG_ACTIVESOURCE, physaddress1, physaddress2)
				logcmd = "** ActiveSourceMessage ** send message: %02X:%02X:%02X to address %02X" % (
					self.MSG_ACTIVESOURCE, physaddress1, physaddress2, AddressValue)
				self.cecdelayedmessage_queue.append((cmd, AddressValue, logcmd))
				self.ActiveSource = True
			elif message == "standby":
				AddressValue = self.ADDR_TV
				cmd = struct.pack('B', self.MSG_STANDBY)
				logcmd = "** StandByMessage ** send message: %02X to address %02X" % (
					self.MSG_STANDBY, AddressValue)
			elif message == "inactive":
				AddressValue = self.ADDR_BROADCAST
				cmd = struct.pack('BBB', self.MSG_INACTIVESOURCE, physaddress1, physaddress2)
				logcmd = "** InActiveSourceMessage ** send message: %02X:%02X:%02X to address %02X" % (
					self.MSG_INACTIVESOURCE, physaddress1, physaddress2, AddressValue)
				self.ActiveSource = False
			elif message == "avpwron":
				cmd = struct.pack('BB', self.MSG_SENDKEY, self.MSG_SENDKEYPWRON)
				AddressValue = self.ADDR_AUDIOSYSTEM
				logcmd = "** Power on A/V ** send message: %02X:%02X to address %02X" % (
					self.MSG_SENDKEY, self.MSG_SENDKEYPWRON, AddressValue)
			elif message == "avaudioon":
				cmd = struct.pack('BBB', self.MSG_AUDIOREQ, avphysaddress1, avphysaddress2)
				AddressValue = self.ADDR_AUDIOSYSTEM
				logcmd = "** Audio on A/V ** send message: %02X:%02X:%02X to address %02X" % (
					self.MSG_AUDIOREQ, avphysaddress1, avphysaddress2, AddressValue)
				self.cecdelayedmessage_queue.append((cmd, AddressValue, logcmd))
			elif message == "avdeeppwroff":
				cmd = struct.pack('BB', self.MSG_SENDKEY, self.MSG_SENDKEYPWROFF)
				AddressValue = self.ADDR_AUDIOSYSTEM
				logcmd = "** Standby A/V (DeepStandby)** send message: %02X:%02X to address %02X" % (
					self.MSG_SENDKEY, self.MSG_SENDKEYPWROFF, AddressValue)
			elif message == "avpwroff":
				AddressValue = self.ADDR_AUDIOSYSTEM
				cmd = struct.pack('BB', self.MSG_SENDKEY, self.MSG_SENDKEYPWROFF)
				logcmd = "** Standby A/V ** send message: %02X:%02X to address %02X" % (
					self.MSG_SENDKEY, self.MSG_SENDKEYPWROFF, AddressValue)
			elif message == "activevu":
				AddressValue = self.ADDR_BROADCAST
				cmd = struct.pack('B', self.MSG_REQUESTACTIVESOURCE)
				logcmd = "** Request active source (Active VU Message) ** send message: %02X to address %02X" % (
					self.MSG_REQUESTACTIVESOURCE, AddressValue)
				self.cecdelayedmessage_queue.append((cmd, AddressValue, logcmd))
			elif message == "physaddress":
				AddressValue = self.ADDR_BROADCAST
				cmd = struct.pack('BBBB', self.MSG_PHYSADDRESS, physaddress1, physaddress2, self.TYPE_RECORDING)
				logcmd = "** Report physical address %02X:%02X:%02X:%02X to %02X" % (
					self.MSG_PHYSADDRESS, physaddress1, physaddress2, self.TYPE_RECORDING, AddressValue)
				self.cecdelayedmessage_queue.append((cmd, AddressValue, logcmd))
			elif message == "discoverdevices":
				self.discoverDevices()
			elif message == "setdevicename":
				AddressValue = self.ADDR_TV
				cecmessage = self.MSG_SETNAME
				name_len = len(config.hdmicec.device_name.value)
				if name_len == 0:
					cecmessagetwo = "VU+"
					cmd = struct.pack('B4s', cecmessage, cecmessagetwo)
				else:
					cecmessagetwo = config.hdmicec.device_name.value
					cmd = struct.pack('B'+str(name_len+1)+'s', cecmessage, cecmessagetwo)
				logcmd = "** Set device name ** send message: %02X:%s to %02X" % (cecmessage, cecmessagetwo, AddressValue)
				self.cecdelayedmessage_queue.append((cmd, AddressValue, logcmd))

			self.delayed_Message_Timer = eTimer()
			self.delayed_Message_Timer.start(20000, True)
			self.delayed_Message_Timer.callback.append(self.delayedMessages)

			if cmd and logcmd:
				self.cecmessage_queue.append((cmd, AddressValue, logcmd))
		if not delay:
			self.sendCECMessage(delay=False)
		else:
			self.delayTimer.start(self.delayTimer_intervall, True)

	def sendCECMessage(self, delay=True):
		try:
			self.delayTimer.stop()
			if len(self.cecmessage_queue):
				cmd, AddressValue, logcmd = self.cecmessage_queue.pop(0)
				self.sendCecMsg(AddressValue, len(cmd), str(cmd))
				if config.hdmicec.logenabledserial.value or config.hdmicec.logenabledfile.value:
					self.doLog(logcmd)
				if not delay:
					messagedelay = float(config.hdmicec.message_delay.value) / 10.0
					time.sleep(messagedelay)
					self.sendCECMessage(delay=False)
				else:
					self.delayTimer.start(self.delayTimer_intervall, True)
		except Exception as E:
			self.doLog("[HDMI-CEC] sendCECMessage: " + E.message)

	def sendCecMsg(self, addr, length, cmd):
		try:
			eHdmiCEC.getInstance().sendMessage(addr, len(cmd), str(cmd))
			if config.hdmicec.logenabledserial.value or config.hdmicec.logenabledfile.value:
				cmdtxt = self.msgtostr("", length, cmd)
				addrtxt = "%02X" % addr
				addrtxt = "%X%s" % (self.ADDR_SELF, addrtxt[1:])
				cec_o_matic = "%s:%s" % (addrtxt, cmdtxt)
				logcmd = "send cec message %s to %02X (%s)" % (cmdtxt, addr, cec_o_matic)
				self.doLog(logcmd)
		except Exception as E:
			self.doLog("[HDMI-CEC] sendCecMsg: " + E.message)

	def delayedMessages(self):

		from Screens.Standby import inStandby

		if not inStandby:
			self.delayTimer.stop()
			while len(self.cecdelayedmessage_queue) > 0:
				cmd, AddressValue, logcmd = self.cecdelayedmessage_queue.pop(0)
				self.cecmessage_queue.append((cmd, AddressValue, "(delayed) " + logcmd))

			if not self.delayTimer.isActive():
				self.delayTimer.start(self.delayTimer_intervall, True)

	def leaveStandby(self):
		if config.hdmicec.enabled.value:
			self.activateSourceTimer()
			msg = str(config.hdmicec.wakeupstandby_message.value)
			if not msg.find('nothing') != -1:
				msg += ",physaddress,setdevicename,discoverdevices"
			self.sendMessages(msg)

	def enterStandby(self, config_element):
		if hasattr(self, "activeSourceTimer"):
			self.activeSourceTimer.stop()
		from Screens.Standby import inStandby
		inStandby.onClose.append(self.leaveStandby)
		if config.hdmicec.enabled.value:
			self.sendMessages(config.hdmicec.standby_message.value)
		if inStandby is not None:
			self.sendMessages("inactive")

	def enterDeepStandby(self, config_element):
		if config.hdmicec.enabled.value:
			self.sendMessages(config.hdmicec.deepstandby_message.value, delay=False)

	def leaveDeepStandby(self):
		if config.hdmicec.enabled.value:
			msg = str(config.hdmicec.wakeupdeepstandby_message.value)
			if not msg.find('nothing') != -1:
				msg += ",physaddress,setdevicename,discoverdevices"
			if not getFPWasTimerWakeup():
				self.sendMessages(msg)
			else:
				if config.hdmicec.disabletimerwakeup.value:
					if config.hdmicec.logenabledserial.value or config.hdmicec.logenabledfile.value:
						self.doLog("timer wakeup => do not power on TV / A/V receiver")
				else:
					self.sendMessages(msg)

	def activateSourceTimer(self):
		self.initial_active_source_call = True
		self.activeSourceTimer = eTimer()
		self.activeSourceTimer.callback.append(self.setActiveSourceCall)
		if not config.hdmicec.active_source_reply.value:
			self.activeSourceTimer.start(60000, True)

	def setActiveSourceCall(self):
		self.initial_active_source_call = False

	@staticmethod
	def hexstrtoint(hexstr, minval, maxval, default=0):
		val = default
		try:
			val = int(hexstr, 16)
			if val not in range(minval, maxval + 1):
				val = default
		except ValueError:
			val = default
		finally:
			return val

	def strtomsg(self, msgtxt):
		msg = msgtxt.split(":")
		if len(msg) > 0:
			msg[0] = "0" + msg[0][1:]  # mask out source address
			addr = self.hexstrtoint(msg[0], 0, 255, 255)
		else:
			addr = 255
		if len(msg) > 1:
			data = []
		else:
			data = [0]
		for i in range(1, len(msg)):
			data.append(self.hexstrtoint(msg[i], 0, 255, 0))
		cmd = struct.pack('%sB' % len(data), *data)
		return addr, data, cmd

	@staticmethod
	def msgdatatostr(data=None):
		strdata = [format(i, '02X') for i in data]
		ret = ":".join(strdata)
		return ret

	def msgtostr(self, addr, length, msg):
		data = struct.unpack('%sB' % length, msg)
		if len(addr) > 0:
			ret = "%02X:%s" % (addr, self.msgdatatostr(data))
		else:
			ret = "%s" % self.msgdatatostr(data)
		return ret
