# -*- coding: utf-8 -*-

# This code is free software, you are allowed to
# modify it (if you keep the license),
# but you are not allowed to distribute/publish
# it without source code (this version and your modifications).
# This means you also have to distribute
# source code of your modifications.

# Simple HTTP server to integrate a web command interface
# this version is for Python 2.7
# and integrates with the VTi HDMI-CEC plugin

# maintainer: remo66 <dev@running-egg.de>

# Changelog:
# 2021-02-05
# * first implementation
# 2021-02-15
# * restart of HTTP server fixed (explicit socket close)
# 2021-02-24
# * response for "cec_log" modified to fit the general interface design

# Todo:
#  * change command delimiter from "/" to "?" and use parse libs (urlparse, parse_qs)???

import threading
from SimpleHTTPServer import SimpleHTTPRequestHandler as BaseHTTPRequestHandler
from SimpleHTTPServer import SimpleHTTPRequestHandler
import SocketServer
# from socket import SOL_SOCKET, SO_REUSEADDR, SHUT_RDWR
from socket import SHUT_RDWR
import os
import time

VERSION = "0.6"


class GetHandler(BaseHTTPRequestHandler):
    MSG_TITLE_HEAD = "VTi HDMI-CEC"
    MSG_FAVICON = '<link rel="icon" href="pic/favicon.png" sizes="16x16" type="image/png">'
    MSG_HEAD = '''<a href="/"><b>VTi HDMI-CEC HTTP Interface (V %s):</b></a>''' % VERSION
    MSG_HELP = '''
                Syntax: 
                <br> 
                <ul style="list-style: none;">
                    <li> http://&lt;hostname or IP address&gt;:&lt;port&gt;/&lt;command&gt;</li>
                </ul>
                <br>
                Commands:
                <br>
                <ul>
                    <li><a href="/cec_hlp">cec_hlp</a>: show this information</li>
                    <li><a href="/cec_log">cec_log</a>: show HDMI-CEC log file of your VU+ device</li>
                    <li><a href="/cec_lst">cec_lst</a>: show list of HDMI-CEC devices in your configuration</li>
                    <li><a href="/cec_msg=00:00">cec_msg</a>: send HDMI-CEC message from your VU+ device to the given destination</li>
                    <ul>
                        <li> format: ={destination address}:{message id}[:data 1]..[:data n] (all in hex)</li>
                        <li> example: cec_msg=05:72:01 (send System Audio Status ON to Audio System)</li>
                        <li> for more information see: <a href="http://www.cec-o-matic.com">CEC-O-MATIC</a></li>
                        <li> you can use tools like 
                            <a href="https://curl.se/docs/manpage.html">cURL</a> or
                            <a href="http://www.gnu.org/software/wget/manual/html_node/Overview.html#Overview">GNU Wget</a> 
                            to automate things from script/batch files</li>
                    </ul>
                </ul>
               '''
    MSG_DEVS_HEAD = '''
                <style>
                    table, th, td {
                        border: 1px solid black;
                        border-collapse: collapse;
                    }
                    th, td {
                        padding: 5px;
                        text-align: center;
                    }
                    th {
                        background-color: #ccc;
                    }
                </style>
                <table style="width:100%">
                    <caption><b>HDMI-CEC Device List</b></caption>
                    <tr>
                        <th>Log. Addr.</th>
                        <th>Device Name</th>
                        <th>Device Vendor</th>
                        <th>Device Type</th>
                        <th>CEC Version</th>
                        <th>Language</th>
                        <th>Phys. Addr.</th>
                        <th>Power Status</th>
                        <th>Active Source</th>
                    </tr>
                '''
    MSG_DEVS_TAIL = '''
                </table> 
                '''
    server_version = "VTi HDMI-CEC HTTP Interface/%s" % VERSION
    log_file = "hdmicec.log"
    tmpdir = os.path.realpath("/tmp")

    def translate_path(self, path):
        path = SimpleHTTPRequestHandler.translate_path(self, path)
        relpath = os.path.relpath(path, os.getcwd())
        fullpath = os.path.join(self.tmpdir, relpath)
        return fullpath
    
    def sendFavicon(self, name):
        ico = None
        picdir = os.path.realpath(self.server.base_dir + "/pic")
        filename = os.path.join(picdir, name)
        f = open(filename, 'rb')
        if f:
            try:
                ico = f.read()
            finally:
                f.close()
        self.wfile.write(bytes(ico))
        
    def do_GET(self):

        # handle favicon request
        if self.path[-12:-4] == "/favicon":
            self.send_response(200)
            if self.path[-4:] == ".ico":
                self.send_header('Content-type', 'image/x-icon')
            else:
                self.send_header('Content-type', 'image/png')
            self.send_header('Access-Control-Allow-Origin', '*')
            self.end_headers()
            self.sendFavicon(self.path[-11:])
            return

        # baseurl = "%s:%d" % (self.server.server_address[0], self.server.server_address[1])
        cmd = self.path[1:]

        # handle device refresh (redirection)
        if cmd.startswith("cec_lst?"):
            addr = int(cmd[-2:], 16)
            ok = self.server.msg_handler.cec_refeshDev(addr)
            time.sleep(3)
            self.send_response(303)
            # redirpath = "%s/%s" % (baseurl, "cec_lst")
            self.send_header('Location', "/cec_lst")
            self.end_headers()
        # handle all other requests
        else:
            self.send_response(200)
            self.send_header("Content-type", "text/html")
            self.send_header("Server", "VTi HDMI-CEC HTTP Interface")
            self.end_headers()

            plugintxt = "Plugin Version: %s" % self.server.plugin_version
            if (len(cmd) == 0) or cmd.startswith("cec_hlp"):
                title = self.MSG_TITLE_HEAD + ": Help"
                result = self.MSG_HELP
            elif cmd.startswith("cec_msg="):
                title = self.MSG_TITLE_HEAD + ": Message"
                msg = cmd[8:]
                result = "message *%s* send" % msg
                ok = self.server.msg_handler.cec_msg(msg)
                if ok:
                    result += " >>> OK"
                else:
                    result += " >>> FAILED"
            elif cmd.startswith("cec_lst"):
                title = self.MSG_TITLE_HEAD + ": Device List"
                lst = self.server.msg_handler.cec_lst()
                result = self.MSG_DEVS_HEAD
                for key in sorted(lst):
                    result += "<tr>\n"
                    result += '<td><a href="/cec_lst?%02X">%02X</a></td>' % (key, key)
                    result += "<td>%s</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td><td>%s</td>\n" \
                              "</tr>\n" % (lst[key][0], lst[key][1], lst[key][2], lst[key][3],
                                           lst[key][4], lst[key][5], lst[key][6], lst[key][7])
                result += self.MSG_DEVS_TAIL
            elif cmd.startswith("cec_log"):
                title = self.MSG_TITLE_HEAD + ": Log"
                self.path = self.log_file
                html = "<html><head><title>%s</title>%s</head><body>" % (title, self.MSG_FAVICON)
                html += "<p>%s<br>%s<br></p><pre>" % (self.MSG_HEAD, plugintxt)
                self.wfile.write(html.encode())
                f = SimpleHTTPRequestHandler.send_head(self)
                if f:
                    try:
                        self.copyfile(f, self.wfile)
                    finally:
                        f.close()
                html = "</pre></body></html>"
                self.wfile.write(html.encode())
                return
            else:
                title = self.MSG_TITLE_HEAD + ": ERROR"
                result = "invalid command"

            html = "<html><head><title>%s</title>%s</head><body>" % (title, self.MSG_FAVICON)
            html += "<p>%s<br>%s<br><br>%s</p></body></html>" % (self.MSG_HEAD, plugintxt, result)

            self.wfile.write(html.encode())


class TCPServer(SocketServer.TCPServer):
    allow_reuse_address = True


class HTTPServer:

    def __init__(self, msg_handler=None, port=8000, url='', basedir=None, version="unknown"):
        self.port = port
        self.thread = None
        self.httpd = None
        self.run = False
        self.url = url
        self.msg_handler = msg_handler
        self.base_dir = basedir
        self.plugin_version = version

    def start(self, port=8000):
        try:
            self.port = port
            self.run = True
            self.httpd = TCPServer((self.url, self.port), GetHandler)
            self.httpd.msg_handler = self.msg_handler
            self.httpd.base_dir = self.base_dir
            self.httpd.plugin_version = self.plugin_version
            # self.httpd.socket.setsockopt(SOL_SOCKET, SO_REUSEADDR, 1)
            self.thread = threading.Thread(target=self._serve)
            self.thread.daemon = True
            self.thread.start()
            return "OK"
        except Exception as E:
            return "Failed (%s)" % E.message

    def _serve(self):
        while self.run:
            self.httpd.handle_request()

    def stop(self):
        try:
            if self.run:
                self.run = False
                if self.httpd is not None:
                    self.httpd.socket.shutdown(SHUT_RDWR)
                    self.httpd.socket.close()
                    self.httpd.server_close()
            return "OK"
        except Exception as E:
            return "Failed (%s)" % E.message
