# -*- coding: utf-8 -*-

# This plugin is free software, you are allowed to
# modify it (if you keep the license),
# but you are not allowed to distribute/publish
# it without source code (this version and your modifications).
# This means you also have to distribute
# source code of your modifications.

# maintainer: <plnick@vuplus-support.org>

# modified by Remo (remo66) <dev@running-egg.de>
#
# 2020-03-06
# * A/V audio activation
# * Message reply changed to sender (not TV) for OSD name (0x46)
# * Logging improved
# * Testing reactivated (press "Play" key to send test message)
# 2021-01-07
# * Logging improved (message payload added / some fixes)
# * CEC device discovery added ("getdevicelist")
# 2021-01-21
# * more exception handling added for stability
# * Configuration item information/help added
# * Testing menu entries improved and moved to end of menu
# 2021-02-02
# * device list window (class DeviceList) added (press [menu] key to open device list)
# 2021-02-05
# * HTTP server added (commands: cec_msg=<cec message> | cec_lst | cec_log | cec_hlp)
# * logging of raw message format (cec-o-matic) added to "MessageReceived"
# 2021-02-15
# * HTML screens improved
# * autostart of HTTP server implemented
# 2021-02-16
# * device list extended (added type, language, phys. addr., active source, power)
# * device discovery extended to retrieve added values in device list
# * device list management optimized
# 2021-02-18
# * device list improved (field order, pixmaps and refresh selected device with [OK] button)
# * configuration menu retructured (Show/Hide Expert/Testing)
# * source code review (lot of code cleaning/ structure fixing)
# 2021-02-21
# * added support to refresh a device from device list in HTTP interface
# * buttons (Close/Update) added to device list window
# 2021-02-24
# * function "keySend" (to send test message) completely rewritten (less source code and unlimited number of data words)
# * replaced all local message variables in function "messageReceived" by "global" constants in class "HdmiCec"

# Todo:
#  * update translation
#  * create github repository???

import struct
from enigma import eActionMap, eHdmiCEC, eTimer
from Plugins.Plugin import PluginDescriptor
from Components.ActionMap import ActionMap, NumberActionMap
from Components.config import config, getConfigListEntry, ConfigInteger, ConfigSubsection, ConfigSelection, \
	ConfigText, ConfigYesNo, NoSave, ConfigNothing, ConfigInteger
from Components.ConfigList import ConfigListScreen
from Components.InputDevice import iInputDevices
from Components.Sources.StaticText import StaticText
from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Tools import Notifications
from Components.Label import Label
from Components.Sources.List import List
from Components.Pixmap import Pixmap
from Tools.LoadPixmap import LoadPixmap
from Tools.Directories import resolveFilename, SCOPE_PLUGINS, SCOPE_CONFIG, SCOPE_CURRENT_SKIN, SCOPE_CURRENT_PLUGIN
import os
from HdmiCec import HdmiCec
from HttpSrv import HTTPServer

VERSION = "RP210224"

hdmi_cec = HdmiCec()


class HTTPcec:

	def __init__(self):
		pass

	@staticmethod
	def cec_refeshDev(addr):
		if config.hdmicec.logenabledserial.value or config.hdmicec.logenabledfile.value:
			hdmi_cec.doLog("device discovery requested for %02X from HTTP interface..." % addr)
		try:
			hdmi_cec.discoverDevice(addr)
			return True
		except Exception as E:
			hdmi_cec.doLog("[HDMI-CEC] cec_refeshDev: " + E.message)
			return False

	@staticmethod
	def cec_lst():
		if config.hdmicec.logenabledserial.value or config.hdmicec.logenabledfile.value:
			hdmi_cec.doLog("device list requested from HTTP interface...")
		try:
			lst = hdmi_cec.DevList
			return lst
		except Exception as E:
			hdmi_cec.doLog("[HDMI-CEC] cec_lst: " + E.message)
			return {}

	@staticmethod
	def cec_msg(msgtxt):
		if config.hdmicec.logenabledserial.value or config.hdmicec.logenabledfile.value:
			hdmi_cec.doLog("message received from HTTP interface: %s" % msgtxt)
		addr, data, cmd = hdmi_cec.strtomsg(msgtxt)
		datatxt = hdmi_cec.msgdatatostr(data)
		addrtxt = "%02X" % addr
		addrtxt = "%X%s" % (hdmi_cec.ADDR_SELF, addrtxt[1:])
		cec_o_matic = "%s:%s" % (addrtxt, datatxt)
		logcmd = "send message from HTTP interface: %s to %02X (%s)" % (datatxt, addr, cec_o_matic)
		# self.session.open(MessageBox, (_("* HDMI-CEC HTTP Message * \n\n%s" % logcmd)), MessageBox.TYPE_INFO, timeout=5)

		try:
			if cmd:
				hdmi_cec.sendCecMsg(addr, len(cmd), str(cmd))

			if logcmd:
				if config.hdmicec.logenabledserial.value or config.hdmicec.logenabledfile.value:
					hdmi_cec.doLog(logcmd)
			return True
		except Exception as E:
			hdmi_cec.doLog("[HDMI-CEC] cec_msg: " + E.message)
			return False


plugin_dir = resolveFilename(SCOPE_CURRENT_PLUGIN, "SystemPlugins/HDMICEC")

http_cec = HTTPcec()
http_srv = HTTPServer(http_cec, basedir=plugin_dir, version=VERSION)


class HdmiCecPlugin(Screen, ConfigListScreen):
	def __init__(self, session):
		# modification in "/usr/share/enigma2/<current_skin>/skin.xml" required to see help
		# <widget name="config" zPosition="2" position="70,115" size="700,420"...
		# <widget source="help" render="Label" position="70,550" size="700,70" font="Regular;20" backgroundColor="background" transparent="1" foregroundColor="labeltext"/>
		self.skin = """
		<screen name="HdmiCecPlugin" position="center,center" size="600,430">
			<ePixmap position="0,5" size="140,40" pixmap="skin_default/buttons/red.png" transparent="1" alphatest="on" />
			<ePixmap position="140,5" size="140,40" pixmap="skin_default/buttons/green.png" transparent="1" alphatest="on" />
			<ePixmap position="280,5" size="140,40" pixmap="skin_default/buttons/yellow.png" transparent="1" alphatest="on" />
			<ePixmap position="420,5" size="140,40" pixmap="skin_default/buttons/blue.png" transparent="1" alphatest="on" />
	
			<widget source="key_red" render="Label" position="0,5" zPosition="1" size="140,40" valign="center" halign="center" font="Regular;21" transparent="1" foregroundColor="white" shadowColor="black" shadowOffset="-1,-1" />
			<widget source="key_green" render="Label" position="140,5" zPosition="1" size="140,40" valign="center" halign="center" font="Regular;21" transparent="1" foregroundColor="white" shadowColor="black" shadowOffset="-1,-1" />
			<widget source="key_yellow" render="Label" position="280,5" zPosition="1" size="140,40" valign="center" halign="center" font="Regular;21" transparent="1" foregroundColor="white" shadowColor="black" shadowOffset="-1,-1" />
			<widget source="key_blue" render="Label" position="420,5" zPosition="1" size="140,40" valign="center" halign="center" font="Regular;21" transparent="1" foregroundColor="white" shadowColor="black" shadowOffset="-1,-1" />
	
			<widget name="config" position="5,50" size="590,275" scrollbarMode="showOnDemand" />
			<ePixmap pixmap="skin_default/div-h.png" position="0,335" zPosition="1" size="565,2" />
			<widget source="help" render="Label" position="5,345" size="590,83" font="Regular;21" />
		</screen>"""

		Screen.__init__(self, session)
		self.session = session
		self.setup_title = _("VTi HDMI-CEC")

		config.hdmicec.avvolup = NoSave(ConfigNothing())
		config.hdmicec.avvoldown = NoSave(ConfigNothing())
		config.hdmicec.avvolmute = NoSave(ConfigNothing())
		config.hdmicec.avpwroff = NoSave(ConfigNothing())
		config.hdmicec.avpwron = NoSave(ConfigNothing())
		config.hdmicec.tvpwroff = NoSave(ConfigNothing())
		config.hdmicec.tvpwron = NoSave(ConfigNothing())
		config.hdmicec.showdevs = NoSave(ConfigNothing())
		config.hdmicec.refreshdevs = NoSave(ConfigNothing())
		config.hdmicec.testsend = NoSave(ConfigNothing())
		
		self["key_red"] = StaticText(_("Close"))
		self["key_green"] = StaticText(_("Save"))
		self["key_yellow"] = StaticText(_("Connect"))
		self["key_blue"] = StaticText(_("Disconnect"))
		
		self["help"] = StaticText(_(" "))
		
		self["setupActions"] = ActionMap(
			["ShortcutActions", "SetupActions", "NumberActions", "InfobarSeekActions", "MenuActions"],
		# self["shortcuts"] = ActionMap(
		# 	["ShortcutActions", "SetupActions", "NumberActions", "InfobarSeekActions", "HelpActions"],
		# self["shortcuts"] = ActionMap(
		# 	["ShortcutActions", "SetupActions", "NumberActions"],	# default
			{
				"ok": self.keyOk,
				"cancel": self.keyCancel,
				"red": self.keyCancel,
				"green": self.keySave,
				"yellow": self.keyConnect,
				"blue": self.keyDisconnect,
				"playpauseService": self.keySend,  # send test message
				"menu": self.showDeviceList,  # open device list
				# "displayHelp": self.showDeviceList, # conflict with "ShootYourScreen" plugin
			}, -2)
		self.list = []
		ConfigListScreen.__init__(self, self.list, session=self.session)

		if self.updateHelp not in self["config"].onSelectionChanged:
			self["config"].onSelectionChanged.append(self.updateHelp)

		self.createSetup()

		self.onLayoutFinish.append(self.setCustomTitle)

	def createSetup(self):
		self.list = []
		self.hdmienabled = getConfigListEntry(_(_("HDMI CEC enabled:")), config.hdmicec.enabled, _("Enable/Disable HDMI Consumer Electronics Control on your VU+"))
		self.hdmidisabledtimerwakeup = getConfigListEntry(_(_("Disable power on actions for timer:")), config.hdmicec.disabletimerwakeup, _("Activate/Deactivate actions on timer events"))
		self.hdmiactivesourcereply = getConfigListEntry(_(_("Active Source Reply On:")), config.hdmicec.active_source_reply, _("Activate/Deactivate reply to active source messages from other devices"))
		self.hdmitvstandby = getConfigListEntry(_("VU+ standby => TV activity:"), config.hdmicec.standby_message, _("Choose TV activity when VU+ turns off to standby"))
		self.hdmitvdeepstandby = getConfigListEntry(_("VU+ deepstandby => TV activity:"), config.hdmicec.deepstandby_message, _("Choose TV activity when VU+ turns off to deep standby"))
		self.hdmitvwakeupstandby = getConfigListEntry(_("VU+ on from standby => TV activity:"), config.hdmicec.wakeupstandby_message, _("Choose TV activity when VU+ turns on from standby"))
		self.hdmitvwakeupdeepstandby = getConfigListEntry(_("VU+ on from deepstandby => TV activity:"), config.hdmicec.wakeupdeepstandby_message, _("Choose TV activity when VU+ turns on from deep standby"))
		self.hdmivustandby = getConfigListEntry(_("TV standby => VU+ activity:"), config.hdmicec.vustandby_message, _("Choose VU+ activity when TV turns off"))
		self.hdmivuwakeup = getConfigListEntry(_("TV on => VU+ activity:"), config.hdmicec.vuwakeup_message, _("Choose VU+ activity when TV turns on"))
		self.hdmitvinput = getConfigListEntry(_("Choose TV HDMI input:"), config.hdmicec.tvinput, _("Choose TV HDMI input"))
		self.hdmiavinput = getConfigListEntry(_("Choose A/V-Receiver HDMI input:"), config.hdmicec.avinput, _("Choose A/V Receiver HDMI input"))
		# self.hdmiavonwakeup = getConfigListEntry(_("Activate A/V-Receiver output on wakeup:"), config.hdmicec.avonwakeup, _("Activate/Deactivate A/V Receiver output on VU+ wakeup"))
		self.hdmiavvolumecontrol = getConfigListEntry(_("Use A/V-Receiver for volume control:"), config.hdmicec.avvolumecontrol, _("Activate/Deactivate forwarding of volume commands to A/V Receiver"))
		self.hdmiavvolup = getConfigListEntry(_("A/V-Receiver volume up:"), config.hdmicec.avvolup, _("A/V Receiver volume down"))
		self.hdmiavvoldown = getConfigListEntry(_("A/V-Receiver volume down:"), config.hdmicec.avvoldown, _("A/V Receiver volume up"))
		self.hdmiavvolmute = getConfigListEntry(_("A/V-Receiver toggle mute:"), config.hdmicec.avvolmute, _("Switch between mute/unmute of A/V Receiver"))
		self.hdmiavpwron = getConfigListEntry(_("A/V-Receiver power on:"), config.hdmicec.avpwron, _("Switch A/V Receiver power on"))
		self.hdmiavpwroff = getConfigListEntry(_("A/V-Receiver power off:"), config.hdmicec.avpwroff, _("Switch A/V Receiver power off"))
		self.hdmitvpwron = getConfigListEntry(_("TV power on:"), config.hdmicec.tvpwron, _("Switch TV power on"))
		self.hdmitvpwroff = getConfigListEntry(_("TV power off:"), config.hdmicec.tvpwroff, _("Switch TV power off"))
		self.hdmienabletvrc = getConfigListEntry(_("Use TV remotecontrol:"), config.hdmicec.enabletvrc, _("This enables/disables support for your TV remote control"))
		self.hdmidevicename = getConfigListEntry(_("Set VU+ device name:"), config.hdmicec.device_name, _("Define the name of your VU+ device which will be reported to other devices in your configuration"))
		self.hdmimessagedelay = getConfigListEntry(_("Delay between CEC messages :"), config.hdmicec.message_delay, _("Set delay between HDMI CEC messages to other devices"))
		self.hdmishowdevs = getConfigListEntry(_("Show device list [Menu]:"), config.hdmicec.showdevs, _("Show a list of HDMI-CEC devices in your configuration"))
		self.hdmirefreshdevs = getConfigListEntry(_("Refresh device list:"), config.hdmicec.refreshdevs, _("Refresh the list of HDMI-CEC devices in your configuration"))
		self.expertenabled = getConfigListEntry(_("Expert settings :"), config.hdmicec.expert_enabled, _("Show or hide settings for debugging, testing and advanced usage"))
		self.hdmilogenabledfile = getConfigListEntry(_("Enable debug output to file :"), config.hdmicec.logenabledfile, _("Activate/Deactivate output to log file"))
		self.hdmilogenabledserial = getConfigListEntry(_("Enable debug output to console :"), config.hdmicec.logenabledserial, _("Activate/Deactivate logging to console"))
		self.httpenabled = getConfigListEntry(_("Enable HTTP interface :"), config.hdmicec.http_enabled, _("Activate/Deactivate HDMI-CEC HTTP interface (use with http://<IP address or hostname of your VU>:<port>/command)\ncommands: cec_hlp | cec_msg=<cec message> | cec_lst | cec_log"))
		self.httpport = getConfigListEntry(_("HTTP port :"), config.hdmicec.http_port, _("Set network port for HDMI-CEC HTTP interface (use with http://<IP address or hostname of your VU>:<port>/command)\ncommands: cec_hlp | cec_msg=<cec message> | cec_lst | cec_log"))
		self.testingenabled = getConfigListEntry(_("Testing :"), config.hdmicec.testing_enabled, _("Show or hide settings to send HDMI-CEC test messages"))
		self.hdmitestaddr = getConfigListEntry(_("Device Address (0..F):"), config.hdmicec.testaddr, _("Destination address for the CEC test message (Send message by pressing [Play] key on your remote or press [Menu] key to get a device list)"))
		self.hdmitestmsg = getConfigListEntry("Message ID (2 hex digits):", config.hdmicec.testmsg, _("Message ID which will define the operation to perform on the destination device (Send message by pressing [Play] key on your remote)"))
		self.hdmitestdata = getConfigListEntry("Message Data (optional):", config.hdmicec.testdata, _("Data of the message as pairs of hex digits delimited by a colon (it's optional and depends on the message"))
		self.hdmitestsend = getConfigListEntry("Send test message [Play]:", config.hdmicec.testsend, _("Send test message with the parameters above"))

		self.list.append(self.hdmienabled)
		if config.hdmicec.enabled.value is True:
			self.list.append(self.hdmidevicename)
			self.list.append(self.hdmitvinput)
			self.list.append(self.hdmiavinput)
			self.list.append(self.hdmivuwakeup)
			self.list.append(self.hdmivustandby)
			self.list.append(self.hdmitvwakeupstandby)
			self.list.append(self.hdmitvwakeupdeepstandby)
			self.list.append(self.hdmitvstandby)
			self.list.append(self.hdmitvdeepstandby)
			self.list.append(self.hdmidisabledtimerwakeup)
			self.list.append(self.hdmienabletvrc)
			if config.hdmicec.avinput.value is not "0":
				# self.list.append(self.hdmiavonwakeup)
				self.list.append(self.hdmiavvolumecontrol)
				self.list.append(self.hdmiavvolup)
				self.list.append(self.hdmiavvoldown)
				self.list.append(self.hdmiavvolmute)
				self.list.append(self.hdmiavpwron)
				self.list.append(self.hdmiavpwroff)
			self.list.append(self.hdmitvpwron)
			self.list.append(self.hdmitvpwroff)
			self.list.append(self.hdmishowdevs)
			self.list.append(self.hdmirefreshdevs)
			self.list.append(self.expertenabled)
			if config.hdmicec.expert_enabled.value is True:
				self.list.append(self.hdmiactivesourcereply)
				self.list.append(self.hdmimessagedelay)
				self.list.append(self.httpenabled)
				self.list.append(self.httpport)
				self.list.append(self.hdmilogenabledserial)
				self.list.append(self.hdmilogenabledfile)
				self.list.append(self.testingenabled)
				if config.hdmicec.testing_enabled.value is True:
					self.list.append(self.hdmitestaddr)
					self.list.append(self.hdmitestmsg)
					self.list.append(self.hdmitestdata)
					self.list.append(self.hdmitestsend)
		self["config"].list = self.list
		self["config"].l.setList(self.list)

	def setCustomTitle(self):
		self.setTitle(' '.join((_("VTi HDMI-CEC"), "(Version", VERSION, ")")))
		
	def updateHelp(self):
		try:
			cur = self["config"].getCurrent()
			if cur:
				self["help"].text = cur[2]
		except Exception as E:
			hdmi_cec.doLog("[HDMI-CEC] updateHelp: " + E.message)
			
	def showDeviceList(self):
		self.session.open(DeviceList)

	@staticmethod
	def keyConnect():
		try:
			hdmi_cec.activateSourceTimer()
			address = 0
			message = 0x85
			messageReceived(None, address, message)
		except Exception as E:
			hdmi_cec.doLog("[HDMI-CEC] keyConnect: " + E.message)

	@staticmethod
	def keyDisconnect():
		physaddress1 = int("0x" + str(config.hdmicec.tvinput.value) + str(config.hdmicec.avinput.value), 16)
		physaddress2 = int("0x00", 16)
		address = int('0', 16)
		cecmessage = int('0x9D', 16)
		cecmessagetwo = physaddress1
		cecmessagethree = physaddress2
		cmd = struct.pack('BBB', cecmessage, cecmessagetwo, cecmessagethree)
		logcmd = "send cec message (Disconnect) %02X:%02X:%02X to %02X" % (cecmessage, cecmessagetwo, cecmessagethree, address)
		
		try:
			if cmd:
				hdmi_cec.sendCecMsg(address, len(cmd), str(cmd))
			
			if logcmd:
				if config.hdmicec.logenabledserial.value or config.hdmicec.logenabledfile.value:
					hdmi_cec.doLog(logcmd)
		except Exception as E:
			hdmi_cec.doLog("[HDMI-CEC] keyDisconnect: " + E.message)

# only used for testing
	def keySend(self):
		try:
			msgtxt = config.hdmicec.testaddr.value
			if len(config.hdmicec.testmsg.value) > 0:
				msgtxt += ":%s" % config.hdmicec.testmsg.value
				if len(config.hdmicec.testdata.value) > 0:
					msgtxt += ":%s" % config.hdmicec.testdata.value
			addr, data, cmd = hdmi_cec.strtomsg(msgtxt)
			datatxt = hdmi_cec.msgdatatostr(data)
			addrtxt = "%02X" % addr
			addrtxt = "%X%s" % (hdmi_cec.ADDR_SELF, addrtxt[1:])
			cec_o_matic = "%s:%s" % (addrtxt, datatxt)
			logcmd = "* Test Message * Send message: %s to %02X (%s)" % (datatxt, addr, cec_o_matic)

			self.session.open(MessageBox, (
				_("* HDMI-CEC Test Message * \n\nSend message: %s to %02X (%s)" % (datatxt, addr, cec_o_matic))),
							MessageBox.TYPE_INFO, timeout=5)

			if cmd:
				hdmi_cec.sendCecMsg(addr, len(cmd), str(cmd))
			
			if logcmd:
				if config.hdmicec.logenabledserial.value or config.hdmicec.logenabledfile.value:
					hdmi_cec.doLog(logcmd)
		except Exception as E:
			hdmi_cec.doLog("[HDMI-CEC] keySend: " + E.message)

# end testing

	def keyOk(self):
		cmd = None
		logcmd = None
		if self["config"].getCurrent() == self.hdmiavvolup:
			address = int("5", 16)
			cecmessage = int("44", 16)
			cecmessagetwo = int("41", 16)
			cmd = struct.pack('BB', cecmessage, cecmessagetwo)
			logcmd = "send cec message %02X:%02X to %02X" % (cecmessage, cecmessagetwo, address)
		elif self["config"].getCurrent() == self.hdmiavvoldown:
			address = int("5", 16)
			cecmessage = int("44", 16)
			cecmessagetwo = int("42", 16)
			cmd = struct.pack('BB', cecmessage, cecmessagetwo)
			logcmd = "send cec message %02X:%02X to %02X" % (cecmessage, cecmessagetwo, address)
		elif self["config"].getCurrent() == self.hdmiavvolmute:
			address = int("5", 16)
			cecmessage = int("44", 16)
			cecmessagetwo = int("43", 16)
			cmd = struct.pack('BB', cecmessage, cecmessagetwo)
			logcmd = "send cec message %02X:%02X to %02X" % (cecmessage, cecmessagetwo, address)
		elif self["config"].getCurrent() == self.hdmiavpwron:
			address = int("5", 16)
			cecmessage = int("44", 16)
			cecmessagetwo = int("6D", 16)
			cmd = struct.pack('BB', cecmessage, cecmessagetwo)
			logcmd = "send cec message %02X:%02X to %02X" % (cecmessage, cecmessagetwo, address)
		elif self["config"].getCurrent() == self.hdmiavpwroff:
			address = int("5", 16)
			cecmessage = int("44", 16)
			cecmessagetwo = int("6C", 16)
			cmd = struct.pack('BB', cecmessage, cecmessagetwo)
			logcmd = "send cec message %02X:%02X to %02X" % (cecmessage, cecmessagetwo, address)
		elif self["config"].getCurrent() == self.hdmitvpwroff:
			address = int("0", 16)
			cecmessage = int("36", 16)
			cmd = struct.pack('B', cecmessage)
			logcmd = "send cec message %02X to %02X" % (cecmessage, address)
		elif self["config"].getCurrent() == self.hdmitvpwron:
			address = int("0", 16)
			cecmessage = int("04", 16)
			cmd = struct.pack('B', cecmessage)
			logcmd = "send cec message %02X to %02X" % (cecmessage, address)
		elif self["config"].getCurrent() == self.hdmishowdevs:
			self.showDeviceList()
		elif self["config"].getCurrent() == self.hdmirefreshdevs:
			hdmi_cec.sendMessages("discoverdevices")
			# hdmi_cec.sendMessages("discoverdevices", False)
			logcmd = "device discovery started from config menu..."
			msg = "Device discovery started.\nIt will take some time to collect all information."
			self.session.open(MessageBox, (_("* HDMI-CEC * \n\n%s" % msg)), MessageBox.TYPE_INFO, timeout=10)
		elif self["config"].getCurrent() == self.hdmitestsend:
			self.keySend()

		else:
			ConfigListScreen.keySave(self)
		try:
			if cmd:
				hdmi_cec.sendCecMsg(address, len(cmd), str(cmd))

			if logcmd:
				if config.hdmicec.logenabledserial.value or config.hdmicec.logenabledfile.value:
					hdmi_cec.doLog(logcmd)
		except Exception as E:
			hdmi_cec.doLog("[HDMI-CEC] keyOk: " + E.message)

	def keyLeft(self):
		try:
			ConfigListScreen.keyLeft(self)
			if self["config"].getCurrent() == self.hdmienabled:
				self.createSetup()
			if self["config"].getCurrent() == self.hdmiavinput:
				self.createSetup()
			if self["config"].getCurrent() == self.expertenabled:
				self.createSetup()
			if self["config"].getCurrent() == self.testingenabled:
				self.createSetup()
			if self["config"].getCurrent() == self.httpenabled:
				switchHTTP(self.session)
		except Exception as E:
			hdmi_cec.doLog("[HDMI-CEC] keyLeft: " + E.message)

	def keyRight(self):
		try:
			ConfigListScreen.keyRight(self)
			if self["config"].getCurrent() == self.hdmienabled:
				self.createSetup()
			if self["config"].getCurrent() == self.hdmiavinput:
				self.createSetup()
			if self["config"].getCurrent() == self.expertenabled:
				self.createSetup()
			if self["config"].getCurrent() == self.testingenabled:
				self.createSetup()
			if self["config"].getCurrent() == self.httpenabled:
				switchHTTP(self.session)
		except Exception as E:
			hdmi_cec.doLog("[HDMI-CEC] keyRight: " + E.message)

	def keyCancel(self):
		try:
			self.close()
		except Exception as E:
			hdmi_cec.doLog("[HDMI-CEC] keyCancel: " + E.message)

	def keySave(self):
		try:
			ConfigListScreen.keySave(self)
		except Exception as E:
			hdmi_cec.doLog("[HDMI-CEC] keySave: " + E.message)


class DeviceList(Screen):

	def __init__(self, session):
		self.skin = """
		<screen name="DeviceList" position="center,center" size="720,350" title="DeviceList">
			<widget source="header0" render="Label" position="10,10" zPosition="1" size="30,40" font="Regular;20" valign="center" halign="center" transparent="1"/>
			<widget source="header1" render="Label" position="40,10" zPosition="1" size="180,40" font="Regular;20" valign="center" halign="center" transparent="1"/>
			<widget source="header2" render="Label" position="220,10" zPosition="1" size="180,40" font="Regular;20" valign="center" halign="center" transparent="1"/>
			<widget source="header3" render="Label" position="400,10" zPosition="1" size="100,40" font="Regular;20" valign="center" halign="center" transparent="1"/>
			<widget source="header4" render="Label" position="500,10" zPosition="1" size="40,40" font="Regular;20" valign="center" halign="center" transparent="1"/>
			<widget source="header5" render="Label" position="540,10" zPosition="1" size="40,40" font="Regular;20" valign="center" halign="center" transparent="1"/>
			<widget source="header6" render="Label" position="580,10" zPosition="1" size="80,40" font="Regular;20" valign="center" halign="center" transparent="1"/>
			<widget source="header7" render="Label" position="660,10" zPosition="1" size="25,40" font="Regular;20" valign="center" halign="center" transparent="1"/>
			<widget source="header8" render="Label" position="685,10" zPosition="1" size="25,40" font="Regular;20" valign="center" halign="center" transparent="1"/>
			<eLabel position="10,48" size="700, 2" zPosition="1" backgroundColor="white" />
			<widget source="devices" render="Listbox" position="10,50" size="700,240" zPosition="1" scrollbarMode="showOnDemand" enableWrapAround="0" transparent="1">
				<convert type="TemplatedMultiContent">
					{"templates":
						{"default": (40,[
								MultiContentEntryText(pos = (0,6), size = (30, 32), flags = RT_HALIGN_CENTER | RT_VALIGN_CENTER, text = 0),
								MultiContentEntryText(pos = (30,6), size = (180, 32), flags = RT_HALIGN_CENTER | RT_VALIGN_CENTER, text = 1),
								MultiContentEntryText(pos = (210,6), size = (180, 32), flags = RT_HALIGN_CENTER | RT_VALIGN_CENTER, text = 2),
								MultiContentEntryText(pos = (390,6), size = (100, 32), flags = RT_HALIGN_CENTER | RT_VALIGN_CENTER, text = 3),
								MultiContentEntryText(pos = (490,6), size = (40, 32), flags = RT_HALIGN_CENTER | RT_VALIGN_CENTER, text = 4),
								MultiContentEntryText(pos = (530,6), size = (40, 32), flags = RT_HALIGN_CENTER | RT_VALIGN_CENTER, text = 5),
								MultiContentEntryText(pos = (570,6), size = (80, 32), flags = RT_HALIGN_CENTER | RT_VALIGN_CENTER, text = 6),
								MultiContentEntryPixmapAlphaBlend(pos = (650, 9), size = (25, 32), png = 7),
								MultiContentEntryPixmapAlphaBlend(pos = (675, 9), size = (25, 32), png = 8),
							]),
						"refresh": (40,[
								MultiContentEntryText(pos = (0,6), size = (30, 32), flags = RT_HALIGN_CENTER | RT_VALIGN_CENTER, text = 0, color_sel = 0x3737ff),
								MultiContentEntryText(pos = (30,6), size = (180, 32), flags = RT_HALIGN_CENTER | RT_VALIGN_CENTER, text = 1, color_sel = 0x3737ff),
								MultiContentEntryText(pos = (210,6), size = (180, 32), flags = RT_HALIGN_CENTER | RT_VALIGN_CENTER, text = 2, color_sel = 0x3737ff),
								MultiContentEntryText(pos = (390,6), size = (100, 32), flags = RT_HALIGN_CENTER | RT_VALIGN_CENTER, text = 3, color_sel = 0x3737ff),
								MultiContentEntryText(pos = (490,6), size = (40, 32), flags = RT_HALIGN_CENTER | RT_VALIGN_CENTER, text = 4, color_sel = 0x3737ff),
								MultiContentEntryText(pos = (530,6), size = (40, 32), flags = RT_HALIGN_CENTER | RT_VALIGN_CENTER, text = 5, color_sel = 0x3737ff),
								MultiContentEntryText(pos = (570,6), size = (80, 32), flags = RT_HALIGN_CENTER | RT_VALIGN_CENTER, text = 6, color_sel = 0x3737ff),
								MultiContentEntryPixmapAlphaBlend(pos = (650, 9), size = (25, 32), png = 7),
								MultiContentEntryPixmapAlphaBlend(pos = (675, 9), size = (25, 32), png = 8),
							]),
						},
						"fonts": [gFont("Regular", 20)],
						"itemHeight": 40
					}
				</convert>
			</widget>
			<widget source="key_red" render="Label" position="10,295" zPosition="1" size="345,40" valign="center" halign="center" font="Regular;21" transparent="1" foregroundColor="white" backgroundColor="red" shadowColor="black" shadowOffset="-1,-1" />
			<widget source="key_green" render="Label" position="365,295" zPosition="1" size="345,40" valign="center" halign="center" font="Regular;21" transparent="1" foregroundColor="white" backgroundColor="green" shadowColor="black" shadowOffset="-1,-1" />
			<eLabel position="10,335" size="345, 5" zPosition="-10" backgroundColor="red" />
			<eLabel position="365,335" size="345, 5" zPosition="-10" backgroundColor="green" />
		</screen>"""

		# <ePixmap position="0,300" size="360,40" pixmap="skin_default/buttons/red.png" transparent="1" alphatest="on" />
		# <ePixmap position="360,300" size="360,40" pixmap="skin_default/buttons/green.png" transparent="1" alphatest="on" />

		Screen.__init__(self, session)
		self["menuActions"] = ActionMap(["OkCancelActions", "ColorActions"],
			{
				"cancel": self.exit,
				"ok": self.startRefresh,
				"red": self.exit,
				"green": self.startRefresh,
			}, -2)
		self['header0'] = StaticText(_("LA"))
		self['header1'] = StaticText(_("Name"))
		self['header2'] = StaticText(_("Vendor"))
		self['header3'] = StaticText(_("Type"))
		self['header4'] = StaticText(_("V"))
		self['header5'] = StaticText(_("Lng"))
		self['header6'] = StaticText(_("PA"))
		self['header7'] = StaticText(_("P"))
		self['header8'] = StaticText(_("S"))
		self["key_red"] = StaticText(_("Close [Exit]"))
		self["key_green"] = StaticText(_("Update Selection [OK]"))
		self.list = []
		self['devices'] = List(self.list)
		self['devices'].style = "default"
		self.refreshidx = 0
		self.RefreshTimer = eTimer()
		self.RefreshTimer.callback.append(self.finishRefresh)
		self.onLayoutFinish.append(self.setWindowTitle)
		self.onLayoutFinish.append(self.fillList)
		if self.selectionChanged not in self["devices"].onSelectionChanged:
			self["devices"].onSelectionChanged.append(self.selectionChanged)

	def setWindowTitle(self):
		self.setTitle(_("HDMI-CEC Devices"))

	def fillList(self):
		imgpath = "SystemPlugins/HDMICEC/pic"
		ps_on = LoadPixmap(resolveFilename(SCOPE_CURRENT_PLUGIN, imgpath + "/on24.png"))
		ps_off = LoadPixmap(resolveFilename(SCOPE_CURRENT_PLUGIN, imgpath + "/off24.png"))
		ps_to_on = LoadPixmap(resolveFilename(SCOPE_CURRENT_PLUGIN, imgpath + "/offon24.png"))
		ps_to_off = LoadPixmap(resolveFilename(SCOPE_CURRENT_PLUGIN, imgpath + "/onoff24.png"))
		active_source = LoadPixmap(resolveFilename(SCOPE_CURRENT_PLUGIN, imgpath + "/active24.png"))

		def pixDirStatus(param):
			if param == hdmi_cec.CECstatus[0x00]:
				ret = ps_on
			elif param == hdmi_cec.CECstatus[0x01]:
				ret = ps_off
			elif param == hdmi_cec.CECstatus[0x02]:
				ret = ps_to_on
			elif param == hdmi_cec.CECstatus[0x03]:
				ret = ps_to_off
			else:
				ret = None
			return ret

		def pixDirActive(param):
			if len(param) > 0:
				return active_source
			else:
				return None

		try:
			lst = hdmi_cec.DevList
			self.list = [("%02X" % key, lst[key][0], lst[key][1], lst[key][2], lst[key][3], lst[key][4], lst[key][5],
						pixDirStatus(lst[key][6]), pixDirActive(lst[key][7])) for key in sorted(lst)]
			self['devices'].setList(self.list)
		except Exception as E:
			hdmi_cec.doLog("[HDMI-CEC] DeviceList.fillList: " + E.message)

	def startRefresh(self):
		try:
			if self['devices'].count > 0:
				self.refreshidx = self['devices'].getIndex()
				self['devices'].style = "refresh"
				self['devices'].updateList(self.list)
				# self['devices'].setIndex(self.refreshidx)
				addr = int(self.list[self.refreshidx][0], 16)
				hdmi_cec.discoverDevice(addr)
				self.RefreshTimer.start(1000, False)
		except Exception as E:
			hdmi_cec.doLog("[HDMI-CEC] DeviceList.startRefresh: " + E.message)

	def finishRefresh(self):
		try:
			self.RefreshTimer.stop()
			self.fillList()
			self['devices'].style = "default"
			self['devices'].setIndex(self.refreshidx)
		except Exception as E:
			hdmi_cec.doLog("[HDMI-CEC] DeviceList.finishRefresh: " + E.message)

	def selectionChanged(self):
		if self['devices'].style != "default":
			self['devices'].style = "default"
			self['devices'].updateList(self.list)

	def exit(self):
		self.close()


def openconfig(session, **kwargs):
	try:
		session.open(HdmiCecPlugin)
	except Exception as E:
		hdmi_cec.doLog("[HDMI-CEC] openconfig: " + E.message)


def autostart(reason, **kwargs):
	global session

	try:
		# if kwargs.has_key("session") and reason == 0:  # old style, removed in Python3
		if "session" in kwargs and reason == 0:
			if config.hdmicec.enabled.value:
				session = kwargs["session"]
				if config.hdmicec.avvolumecontrol.value:
					# from InfoBarGenerics.py
					eActionMap.getInstance().bindAction('', -0x7FFFFFFF, volumekeyPressed)
					#
				eHdmiCEC.getInstance().cecMessageReceived.get().append(messageReceived)
				eHdmiCEC.getInstance().messageReceivedKey.get().append(messageReceivedKey)
				switchHTTP(session)
	except Exception as E:
		hdmi_cec.doLog("[HDMI-CEC] autostart: " + E.message)


def Plugins(**kwargs):
	return [PluginDescriptor(name=_("HDMI-CEC"), description="VTI HDMI-CEC Configuration", where=PluginDescriptor.WHERE_PLUGINMENU, icon="hdmicec.png", needsRestart=True, fnc=openconfig),
			PluginDescriptor(where=[PluginDescriptor.WHERE_SESSIONSTART, PluginDescriptor.WHERE_AUTOSTART], fnc=autostart)]


def switchHTTP(session):
	http_state = "has unknown status"
	try:
		if config.hdmicec.http_enabled.value is True:
			if not http_srv.run:
				http_port = int(config.hdmicec.http_port.value)
				ret = http_srv.start(http_port)
				http_state = "started on port %d [%s]" % (http_port, ret)
		else:
			if http_srv.run:
				ret = http_srv.stop()
				http_state = "stopped [%s]" % ret
		hdmi_cec.doLog("HTTP interface %s!" % http_state)
		session.open(MessageBox, (
			_("* HDMI-CEC HTTP Interface * \n\n" + "HTTP interface %s!" % http_state)), MessageBox.TYPE_INFO, timeout=5)
	except Exception as E:
		hdmi_cec.doLog("switchHTTP: " + E.message)


def volumekeyPressed(key, flag):
	try:
		if config.hdmicec.avvolumecontrol.value and config.hdmicec.enabled.value and config.hdmicec.avinput.value != "0":
			if key == 113 or key == 114 or key == 115:
				address = int("5", 16)
				cecmessagesendkey = int("44", 16)
				cecmessagekeyevent = None
				if flag == 1:
					cecmessagekeybreak = int("45", 16)
					cmd = struct.pack('B', cecmessagekeybreak)
				else:
					if key == 113:
						cecmessagekeyevent = int("43", 16)
					elif key == 114:
						cecmessagekeyevent = int("42", 16)
					elif key == 115:
						cecmessagekeyevent = int("41", 16)
					if cecmessagekeyevent:
						cmd = struct.pack('BB', cecmessagesendkey, cecmessagekeyevent)
				if cmd:
					hdmi_cec.sendCecMsg(address, len(cmd), str(cmd))
				return 1
		return 0
	except Exception as E:
		hdmi_cec.doLog("[HDMI-CEC] volumekeyPressed: " + E.message)


def messageReceived(cecdata, manual_address=None, manual_cmd=None):
	address = 16 * '\xFF'
	message = 16 * '\x00'
	data = 16 * '\x00'
	datatxt = ""
	length = 0
	if cecdata is not None:
		message = cecdata.getCommand()
		address = cecdata.getAddress()
		length = cecdata.getData(data, len(data))
		for i in range(0, length-1):
			if i == 0:
				datatxt = "%02X" % ord(data[i])
			else:
				datatxt += ":%02X" % ord(data[i])
	elif manual_address is not None and manual_cmd is not None:
		message = manual_cmd
		datatxt = "manual action"
		address = manual_address

	try:
		if address not in hdmi_cec.DevList:
			hdmi_cec.updateDeviceList(address)  # add initial entry for device
			hdmi_cec.discoverDevice(address)  # discover device info

		# log every incoming message, if logging is enabled
		cec_o_matic = "%X%X:%02X" % (address, hdmi_cec.ADDR_SELF, message)
		if len(datatxt) > 0:
			cec_o_matic += ":%s" % datatxt
		logcmd = "received cec message %02X[%s] from %02X (%s)" % (message, datatxt, address, cec_o_matic)
		if config.hdmicec.logenabledserial.value or config.hdmicec.logenabledfile.value:
			hdmi_cec.doLog(logcmd)

		if config.hdmicec.enabled.value:
			from Screens.Standby import inStandby
			from Screens.Standby import Standby

			cmd = None
			cmdtwo = None
			addresstwo = None
			logcmd = None
			logcmdtwo = None

			physaddress1 = int("0x" + str(config.hdmicec.tvinput.value) + str(config.hdmicec.avinput.value), 16)
			physaddress2 = int("0x00", 16)

			if message == 0x00:  # system audio control received
				if length == 0:  # only polling message ( it's like a ping )
					logcmd = "received polling message from %02X" % address
				else:
					# feature abort
					logcmd = "received feature abort (not supported by device): %02X[%s] from %02X" % (message, datatxt, address)
					# if data[0] == '\x44':
					# 	logcmd = "received volume forwarding not supported by device with data %02x from %02x" % (data, address)
					# 	self.volumeForwardingEnabled = False

			elif message == 0x32:  # menu language received
				txt = ""
				for i in range(0, length-1):
					txt += "%s" % data[i]
				logcmd = "menu language [%s] received from %02X" % (txt, address)
				hdmi_cec.updateDeviceList(address, lng=txt)

			elif message == 0x36:  # request sytem standby
				if config.hdmicec.vustandby_message.value == "vustandby":
					if inStandby is None:
						logcmd = "VU+ STB goto standby"
						session.open(Standby)
				elif config.hdmicec.vustandby_message.value == "vudeepstandby":
					import Screens.Standby
					logcmd = "VU+ STB goto deepstandby"
					session.open(Screens.Standby.TryQuitMainloop, 1)

			elif message == 0x46:  # request device name
				# address = addresstv 	#Bug: send answer to sender and not allways to TV
				cecmessage = hdmi_cec.MSG_SETNAME
				name = hdmi_cec.getConfigDeviceName()
				name_len = len(name)
				cecmessagetwo = name
				cmd = struct.pack('B' + str(name_len+1) + 's', cecmessage, cecmessagetwo)
				logcmd = "send device OSD name with cec message %02X:[%s] to %02X" % (cecmessage, cecmessagetwo, address)

			elif message == 0x47:  # device name received
				txt = ""
				for i in range(0, length-1):
					txt += "%s" % data[i]
				logcmd = "device name [%s] received from %02X" % (txt, address)
				hdmi_cec.updateDeviceList(address, name=txt)

			elif message == 0x72:  # system audio control received
				datatxt = "%02X" % ord(data[0])
				logcmd = "system audio mode message: %02X[%s] from %02X" % (message, datatxt, address)

			elif message == 0x82:  # active source received
				data_pieces = datatxt.split(":")
				pa = "%s.%s.%s.%s" % (
					data_pieces[0][0:1], data_pieces[0][1:2], data_pieces[1][0:1], data_pieces[1][1:2])
				logcmd = "active source received (physical address: %s) from %02X" % (pa, address)
				hdmi_cec.updateDeviceList(address, active="X")

			elif message == 0x83:  # request physical address
				if inStandby and (config.hdmicec.vuwakeup_message.value == "vuwakeup"):
					inStandby.Power()
				address = hdmi_cec.ADDR_BROADCAST
				cecmessage = hdmi_cec.MSG_PHYSADDRESS
				cecmessagetwo = physaddress1
				cecmessagethree = physaddress2
				cecmessagefour = hdmi_cec.TYPE_SELF
				cmd = struct.pack('BBBB', cecmessage, cecmessagetwo, cecmessagethree, cecmessagefour)
				logcmd = "send cec message %x:%x:%x:%x to %02X" % (cecmessage, cecmessagetwo, cecmessagethree, cecmessagefour, address)

			elif message == 0x84:  # physical address (and device type) received
				pa = "unknown"
				typetxt = "undefined"
				try:
					data_pieces = datatxt.split(":")
					pa = "%s.%s.%s.%s" % (data_pieces[0][0:1], data_pieces[0][1:2], data_pieces[1][0:1], data_pieces[1][1:2])
					hdmi_cec.updateDeviceList(address, pa=pa)
					devtype = int(data_pieces[2], 16)
					typetxt = hdmi_cec.CECtypes[devtype]
				finally:
					hdmi_cec.updateDeviceList(address, devtype=typetxt)
					logcmd = "physical address and device type reported with: %02X[%s -> %s/%s] from %02X" % (message, datatxt, pa, typetxt, address)

			elif message == 0x85:  # request active source
				if not inStandby:
					if config.hdmicec.active_source_reply.value or hdmi_cec.initial_active_source_call is True:
						address = hdmi_cec.ADDR_BROADCAST
						cecmessage = hdmi_cec.MSG_ACTIVESOURCE
						cecmessagetwo = physaddress1
						cecmessagethree = physaddress2
						cmd = struct.pack('BBB', cecmessage, cecmessagetwo, cecmessagethree)
						logcmd = "send cec message %02X:%02X:%02X to %02X" % (cecmessage, cecmessagetwo, cecmessagethree, address)
						hdmi_cec.ActiveSource = True
					if config.hdmicec.enabletvrc.value:
						addresstwo = hdmi_cec.ADDR_TV
						cecmessage = hdmi_cec.MSG_MENUSTATUS
						cecmessagetwo = hdmi_cec.MENU_ON
						cmdtwo = struct.pack('BB', cecmessage, cecmessagetwo)
						logcmdtwo = "send cec message %02X:%02X to %02X" % (cecmessage, cecmessagetwo, address)
				elif inStandby:
					if config.hdmicec.vuwakeup_message.value == "vuwakeup":
						inStandby.Power()
						address = hdmi_cec.ADDR_BROADCAST
						cecmessage = hdmi_cec.MSG_ACTIVESOURCE
						cecmessagetwo = physaddress1
						cecmessagethree = physaddress2
						cmd = struct.pack('BBB', cecmessage, cecmessagetwo, cecmessagethree)
						logcmd = "send cec message %02X:%02X:%02X to %02X" % (cecmessage, cecmessagetwo, cecmessagethree, address)
						hdmi_cec.ActiveSource = True
						if config.hdmicec.enabletvrc.value:
							addresstwo = hdmi_cec.ADDR_TV
							cecmessage = hdmi_cec.MSG_MENUSTATUS
							cecmessagetwo = hdmi_cec.MENU_ON
							cmdtwo = struct.pack('BB', cecmessage, cecmessagetwo)
							logcmdtwo = "send cec message %02X:%02X to %02X" % (cecmessage, cecmessagetwo, address)

			elif message == 0x86:  # set stream path (routing control)
				physicaladdress = ord(data[0]) * 256 + ord(data[1])
				pysaddrrstr = "%x" % physicaladdress
				confAddress = "%x%02X" % (physaddress1, physaddress2)
				if pysaddrrstr == confAddress:
					if inStandby and (config.hdmicec.vuwakeup_message.value == "vuwakeup"):
						inStandby.Power()
					address = hdmi_cec.ADDR_BROADCAST
					cecmessage = hdmi_cec.MSG_ACTIVESOURCE
					cecmessagetwo = physaddress1
					cecmessagethree = physaddress2
					cmd = struct.pack('BBB', cecmessage, cecmessagetwo, cecmessagethree)
					logcmd = "send cec message %x:%x:%x to %02X" % (cecmessage, cecmessagetwo, cecmessagethree, address)
					hdmi_cec.sendCecMsg(address, len(cmd), str(cmd))
					hdmi_cec.ActiveSource = True
					if config.hdmicec.enabletvrc.value:
						addresstwo = hdmi_cec.ADDR_TV
						cecmessage = hdmi_cec.MSG_MENUSTATUS
						cecmessagetwo = hdmi_cec.MENU_ON
						cmdtwo = struct.pack('BB', cecmessage, cecmessagetwo)
						hdmi_cec.sendCecMsg(addresstwo, len(cmdtwo), str(cmdtwo))
				else:
					logcmd = "received %02X with data %x my conf %s" % (message, physicaladdress, confAddress)

			elif message == 0x87:  # Vendor ID received
				vendortxt = "undefined"
				try:
					datatxt = datatxt.replace(":", "")
					vendorid = int(datatxt, base=16)
					vendortxt = hdmi_cec.CECvendors[vendorid]
				finally:
					hdmi_cec.updateDeviceList(address, vendor=vendortxt)
					logcmd = "device vendor ID [%s -> %s] received from %02X" % (datatxt, vendortxt, address)

			elif message == 0x8C:  # request Vendor ID
				cecmessage = hdmi_cec.MSG_VENDORID
				cecmessagetwo = hdmi_cec.VENDORID_SELF
				cmd = struct.pack('BB', cecmessage, cecmessagetwo)
				logcmd = "send cec message %02X:%02X to %02X" % (cecmessage, cecmessagetwo, address)

			elif message == 0x8d:  # request menu state
				if config.hdmicec.enabletvrc.value:
					address = hdmi_cec.ADDR_TV
					cecmessage = hdmi_cec.MSG_MENUSTATUS
					cecmessagetwo = hdmi_cec.MENU_ON
					cmd = struct.pack('BB', cecmessage, cecmessagetwo)
					logcmd = "send cec message %02X:%02X to %02X" % (cecmessage, cecmessagetwo, address)

			elif message == 0x8F:  # request power status
				# address = addresstv	#Bug: send answer to sender and not allways to TV
				cecmessage = hdmi_cec.MSG_REPORTPOWERSTATUS
				cecmessagetwo = hdmi_cec.POWER_ON
				if inStandby and (config.hdmicec.vuwakeup_message.value == "vuwakeup"):
					cecmessagetwo = hdmi_cec.POWER_OFF
				cmd = struct.pack('BB', cecmessage, cecmessagetwo)
				logcmd = "send cec message %02X:%02X to %02X" % (cecmessage, cecmessagetwo, address)

			elif message == 0x90:  # power status received
				statustxt = "undefined"
				try:
					status = int(datatxt[:2], base=16)
					statustxt = hdmi_cec.CECstatus[status]
				finally:
					hdmi_cec.updateDeviceList(address, power=statustxt)
					logcmd = "device power status [%s -> %s] received from %02X" % (datatxt, statustxt, address)

			elif message == 0x9E:  # device CEC version received
				vertxt = "undefined"
				try:
					ver = int(datatxt[:2], base=16)
					vertxt = hdmi_cec.CECversions[ver]
				finally:
					hdmi_cec.updateDeviceList(address, ver=vertxt)
					logcmd = "device CEC version [%s -> V%s] received from %02X" % (datatxt, vertxt, address)

			elif message == 0x9F:  # request CEC version
				# address = addresstv	#Bug: send answer to sender and not allways to TV
				cecmessage = hdmi_cec.MSG_CECVERSION
				cecmessagetwo = hdmi_cec.CECVER_SELF
				cmd = struct.pack('BB', cecmessage, cecmessagetwo)
				logcmd = "send cec message %02X:%02X to %02X" % (cecmessage, cecmessagetwo, address)

			if inStandby is None:
				if cmd:
					hdmi_cec.sendCecMsg(address, len(cmd), str(cmd))
				if cmdtwo:
					hdmi_cec.sendCecMsg(addresstwo, len(cmdtwo), str(cmdtwo))
				if logcmd:
					if config.hdmicec.logenabledserial.value or config.hdmicec.logenabledfile.value:
						hdmi_cec.doLog(logcmd)
				if logcmdtwo:
					if config.hdmicec.logenabledserial.value or config.hdmicec.logenabledfile.value:
						hdmi_cec.doLog(logcmdtwo)
	except Exception as E:
		hdmi_cec.doLog("[HDMI-CEC] messageReceived: " + E.message)


def messageReceivedKey(address, message):
	try:
		logcmd = "received cec message part two %02X from %02X" % (message, address)
		if logcmd:
			if config.hdmicec.logenabledserial.value or config.hdmicec.logenabledfile.value:
				hdmi_cec.doLog(logcmd)

		if config.hdmicec.enabled.value is True:
			rcdevicename = iInputDevices.getDeviceName('event0')  # hschang : get rc device name, /dev/input/event0
			keyaction = eActionMap.getInstance()
			key = None
			if message == 0x32 or message == 0x09:  # key menu
				key = int(139)
			elif message == 0x20:  # key 0
				key = int(11)
			elif message == 0x21:  # key 1
				key = int(2)
			elif message == 0x22:  # key 2
				key = int(3)
			elif message == 0x23:  # key 3
				key = int(4)
			elif message == 0x24:  # key 4
				key = int(5)
			elif message == 0x25:  # key 5
				key = int(6)
			elif message == 0x26:  # key 6
				key = int(7)
			elif message == 0x27:  # key 7
				key = int(8)
			elif message == 0x28:  # key 8
				key = int(9)
			elif message == 0x29:  # key 10
				key = int(10)
			elif message == 0x30:  # key bouquet up
				key = int(402)
			elif message == 0x31:  # key bouquet down
				key = int(403)
			elif message == 0x53:  # key info/epg
				key = int(358)
			elif message == 0x00:  # key ok
				key = int(352)
			elif message == 0x03:  # key left
				key = int(105)
			elif message == 0x04:  # key right
				key = int(106)
			elif message == 0x01:  # key up
				key = int(103)
			elif message == 0x02:  # key down
				key = int(108)
			elif message == 0x0d:  # key exit
				key = int(174)
			elif message == 0x72:  # key red
				key = int(398)
			elif message == 0x71:  # key blue
				key = int(401)
			elif message == 0x73:  # key green
				key = int(399)
			elif message == 0x74:  # key yellow
				key = int(400)
			elif message == 0x44:  # key play
				if rcdevicename.find("advanced"):
					key = int(164)  # KEY_PLAYPAUSE
				else:
					key = int(207)  # KEY_PLAY
			elif message == 0x46:  # key pause
				if rcdevicename.find("advanced"):
					key = int(164)  # KEY_PLAYPAUSE
				else:
					key = int(119)  # KEY_PAUSE
			elif message == 0x45:  # key stop
				key = int(128)
			elif message == 0x47:  # key record
				key = int(167)
			elif message == 0x49:  # fast forward
				if rcdevicename.find("advanced"):
					key = int(163)  # KEY_NEXTSONG
				else:
					key = int(208)  # KEY_FASTFORWARD
			elif message == 0x48:  # rewind
				if rcdevicename.find("advanced"):
					key = int(165)  # KEY_NEXTSONG
				else:
					key = int(168)  # KEY_FASTFORWARD
			elif message == 0x60:  # play 2
				key = int(207)
			elif message == 0x61:  # key pause 2
				if rcdevicename.find("advanced"):
					key = int(164)  # KEY_PLAYPAUSE
				else:
					key = int(119)  # KEY_PAUSE
			elif message == 0x64:  # key stop 2
				key = int(128)
			elif message == 0x62:  # key record 2
				key = int(167)
			# end translate keycodes
			if key:
				keyaction.keyPressed(rcdevicename, key, int(0))
				keyaction.keyPressed(rcdevicename, key, int(1))
	except Exception as E:
		hdmi_cec.doLog("[HDMI-CEC] messageReceivedKey: " + E.message)
